/*

Euclidean Sequencer for 4U Vers 1.1

used with permission by Tombola and Seak Theif
compatible with hardware bu Clee

Vers 1.1 fixes a bug where setting the TRIG/GATE switch to gate would cause the MIDI out
to not send Note-Offs.


Craig Lee 4/29/18
*/





//Euclidian Pattern Generator
//Main TAB


#include <LedControl.h>
#include <EEPROM.h>
#include <Encoder.h>
//#include <PWM.h>




/* Instructions:

  -- Note: when first turned on, it runs automatically from a fixed speed internal clock until it receives
  a pulse at the clock input, after that it only triggers externally --

  When the rhythms are playing, the display flips between page 1 (steps 1-8) and page 2 (steps 9-16).

  Here's what the display shows...

  Row 1: What step is Output 1 playing
  Row 2: Output 1 Pattern (steps 1 - 8 or 9 - 16)

  Row 3: What step is Output 2 playing
  Row 4: Output 2 Pattern (steps 1 - 8 or 9 - 16)

  Row 5: What step is Output 3 playing
  Row 6: Output 3 Pattern (steps 1 - 8 or 9 - 16)

  Row 7: Which channel is selected

  - 2 dots on the left for Channel 1
  - 2 dots in the middle for Channel 2
  - 2 dots on the right for Channel 3

  Row 8 : Current triggers

    1. Input trigger
    2. -
    3. Output 1 trigger
    4. Output 1 off - beat trigger (when Output 1 isn't playing
    5. Output 2 trigger
    6. -
    7. Output 3 trigger
    8. -

  - When you have Channel 1 selected and you rotate the N- and K- and Offset knobs,
   Rows 1 and 2 will respectively show the pattern length (N), pattern density (K) or Offset (O).

  The same goes for Channel 2 (Rows 3 and 4) and Channel 3 (Rows 5 and 6)

  - Rotating the Offset encoder clockwise rotates the steps up to one full rotation

  Example:

  X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 - Original 16-step pattern (N = 16, K = 1)
  0 X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 - Offset of 1
  0 0 X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 - Offset of 2
  0 0 0 X 0 0 0 0 0 0 0 0 0 0 0 0 0 - Offset of 3
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 X - Offset of 15
  X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 - Offset of 16

  >>> Sneak-Thief's design notes <<<

  - Added Encoder library to stabilize encoder use
  - Cleaned up code a bit
  - Created schematic with input/output protection: http://sneak-thief.com/modular/tombolas-euclidean-0.7.png
  - Added reset gate input
  - Added Offset control
  - Added 10K pulldown resistor to switch outputs
  - Changed 15K resistors to 10K
  - Added more error checking for eeprom reads / values to reduce risk of crashes

  >>> Syinsi/THX2112's design notes <<<

  - Updated schematic and included PCB files. Check distribution folder for files.
  - Uses separate MAX7219 IC and 8x8 matrix's.
  - Transistor-driven outputs.
  - BAT54S dual-schottky input protection.
  - Isolated 1.5A 5V supply for led matrix.

  To do
  - Add CV control of N, K and Offset using analog pins 5,6,7
  - OR Add gate control of two pairs of N, K or Offset using analog pins 4,5,6,7

  >>> Tombola's original design notes <<<

  To do
  - Error checking for eeprom reads / values to reduce risk of crashes
  - Find cause of ocassional skipped beats?

  Done
  - Connect 'off beat' for channel 1 to the spare output
  - something causing channel 1 to stick - N changes don't appear
  - When N is turned down, reduce K accordingly
  - make tick pulse correctly
  - Fix drawing of beats  as beats are playing
  - binary display of K & N not right
  - remove serial print / debug routine
  - remove delay - replace with 'all pulses off after 5ms routine
  - Fix crashing issue with low n
  - channels 1 and 3 require tweak to start running
  - Display OK
  - Pulse in working up to audio rates
  - Got flashing working - system is outputting euclidean beats!
  - Integrate encoders & eeprom
  - Add encoder display - light up row as it's turning DONE
  - 3-way switch working
  - Active channel indicator
  - Sleep / wake system & animation
  - DONE Remove serial printing - slows down?

  TO DO:
  - Implement offset (added by Sneak-Thief)
  - Add reset outputs and input and button
  - Add CV control of N and K (and later Offset) using analog pins 5,6,7

  BUGS:
  - FIXED Still display issues - 1111 on the end of beat displays - seems not to be clearing properly
  - FIXED Display on beats where k=n - seems to show loads of binary 111 on the end
  - FIXED Fix the bottom row - currently output flashers reversed

  Hardware notes:

  Display:
  din = d2
  clk = d3
  load = d4

  Encoders:
  Encoder 1a - k / beats = d5
  Encoder 1b - k / beats = d6
  Encoder 2a - n / length = d7
  Encoder 2b - n / length  = d8
  Encoder 3a - Offset = d9
  Encoder 3 b - offset = d10

  Pulse outputs:
  1 = d11
  2 = d12
  3 = d13

  Switches / misc :
  reset switch = A1
  pulse input = A0
  spare jack out = A3
  encoder switch = A2  - 3 buttons share one analog input:

  10k resistor ladder around 3 push button switches
  +5v -- 10k -- switch -- 10k -- switch -- 10k -- switch -- 10k -- GND
  Other ends of switches going to Analog in 0 and also 10k to GND

  Order: [HIGHEST R] Offset, Length, Density, (unpressed) [LOWEST R]


  //----------------------------------------------------------------------------------------------------

  // CLee additions

  /*  EEPROM changes

  EEPROM is now written only when encoders are moved and when the E_displayDelay or M_displayDelay
  counter counts down (no encoder movement happens for approx 2 seconds). Then all EEPROM Euclidaan or MIDI
  locations are read and the ones that have changed are written to.

*/


/* New MIDI function


  HOW TO SET MIDI VARIABLES

  A long press of encoder 1 switch sets encoders to control MIDI parameters

  the first 4 LEDs in row 7 will show which sequencer channel's MIDI is being updated
  Switch 1 short press steps through 4 sequencer channels

  ENC 1 MIDI CHANNEL for all sequencer channels
  ENC 2 VELOCITY X4 channels
  ENC 3 NOTE X4 channels

  Long press of encoder 1 returns controls to sequencer


  EEPROM ADDED:

  MIDI channel is set for all 4 channels universally, but memory locations are set up so they could be set individually by updating software.
  EEPROM(13) is used for MIDI channel

  EEPROM locations:
  Channel 1  MIDI Channel = 13  Velocity = 17  Note = 21
  Channel 2  MIDI Channel = 14  Velocity = 18  Note = 22
  Channel 3  MIDI Channel = 15  Velocity = 19  Note = 23
  Channel 4  MIDI Channel = 16  Velocity = 20  Note = 24

  Array to store currently running sequence MIDI:

  MIDI_sets[4][2]; // Array to store MIDI note and velocity settings

  ARRAY SETUP
  [0][0] CH 1 Velocity, [0][1] CH 1 Pitch
  [1][0] CH 2 Velocity, [1][1] CH 2 Pitch
  [2][0] CH 3 Velocity, [2][1] CH 3 Pitch
  [3][0] Offb Velocity, [3][1] Offb Pitch


  Commands/Subroutines used
  playMIDINote(); - sends a MIDI byte, status and note_on and velocity bytes
  playMIDIOff();  - sends a MIDI noet_off command with velocity of zero


  -------- CLee added patch storage ----------------------------------------
  there are 8 preset locations for storing sequence data and MIDI setups

  Long press of the Encoder 3 switch sets patch storage or recall
  LED row 7 will show which preset will be recalled or stored to
  LED row 8 changes to a pattern showing the mode

  Encoder 1 will select the preset location (1-8)
  long press of encoder 2 will will read the selected preset into the currently active sequence
  long press of encoder 3 will will write the currently active sequence into the selected preset
  LED row 7 will count up to the read/write by turning on LEDs one at a time

  Encoders will return to the last mode used (Sequencer or MIDI) after the read/write.

  A long press of encoder 1 will exit preset mode without read or write

  Preset storage is currently 25 bytes, incluing the contents of the channelbeats[3][5] and MIDI_sets[4][2] arrays
  The preset size is set by variable: int Preset_size = 25;



  EEPROM MAP

  EEPROM 1-12 are Channel beats locations
  EEPROM 13 -24 are MIDI Channel, Velocity and Note locations
  EEPROM 25 is brighness
  EEPROM 26-49 are unused
  EEPROM 50 - 249 are preset storage (25 * 8 offset by 50)


*/


// Debug Flag
int debug = 1; // 0= normal  1= Internal Clock  2= Internal Clock and SerialDump

// Encoder Pins for the Encoder.h library
Encoder EncK(10, 9);	//	Density
Encoder EncN(8, 7);		//	Length
Encoder EncO(6, 5);   //	Offset

// Encoder Pins for internal use (must correspond to the previous pins)
#define enc1a 10
#define enc1b 9
#define enc2a 7
#define enc2b 8
#define enc3a 5
#define enc3b 6

//#define sparepin 17 // Offbeat pin

LedControl lc = LedControl(2, 3, 4, 1); // Matrix LED pins

#define brightness 0	//	From 0 (low) to 15

#define display_update 2000 // how long active channel display is shown

int pulse_length = 50; //pulse length

unsigned long time;
unsigned long last_sync;

int channels = 3;
unsigned int beat_holder[3];

/*
  Eeprom schema:
  Channel 1: n = 1 k = 2 position = 7
  Channel 2: n = 3 k = 4 position = 8
  Channel 3: n = 5 k = 6 position = 9
*/

// load saved Euclidian settings
unsigned int channelbeats[3][5] =
{
  { EEPROM.read(1), EEPROM.read(2), 0, EEPROM.read(7), EEPROM.read(10)},
  {EEPROM.read(3), EEPROM.read(4), 0, EEPROM.read(8), EEPROM.read(11)},
  {EEPROM.read(5), EEPROM.read(6), 0, EEPROM.read(9), EEPROM.read(12)}
}; // 0=n, 1=k, 2 = position , 3 = offset

int a;
int changes = 0;        // knob changed flag
boolean sleep = true;   // LED Sleep mode enable/disable
int masterclock = 0;    // Internal clock enable/disable
int read_head;
unsigned int  looptracker;

int old_total;     //for knobs
int old_pulses;    //for knobs

int pulseinput = 0;
boolean newpulse;      //for trigger in -CHANGED TO BOOLEAN
boolean oldpulse = 1;  //for trigger in -CHANGED TO BOOLEAN

boolean pulses_active = false; // is active while a beat pulse is playing
boolean lights_active = false;

int kknob;
int active_channel = 1; // which channel is active? zero indexed
int nknob;
int oknob;
int maxn = 16; // maximums and minimums for n and k
int minn = 1;
int mink = 1;
int maxo = 15; // maximums and minimums for o
int mino = 0;
int nn;
int kk;
int oo;

unsigned long last_read;
unsigned long last_changed;
#define read_delay  80 // for debouncing of encoders ****************
int channel_switch;
int reset_button;
int channel_switch_read;



// MIDI VARIABLES

//Display related
int displayDelay = 30000; // loop counter for displaying MIDI updates Set to 3000 if using Analog reads from original code
int M_displayDelay = 1;  // loop counter for MIDI to take over bottom 2 rows and flag a MIDI EEPROM Write (decrimented in M_read_pots loop)
int E_displayDelay = 1;  // loop counter flag for a Euclidian EEPROM Write
unsigned long last_read_M;  // encoder debounce in MIDI
// these control the bottom 2 rows of the LED display and use the "readout" tab
int dots;       // how many dots lit on bar graph display (row 7) for MIDI Channel & Velocity and Preset store/recall
int dots_octave; // octave of midi notes (binary readout) on row 8

// variables for Euclidian/MIDI/Preset function switching
int old_channel_switch;            // state last time through loop - used in long press routine
unsigned long long_press;          // time the long button press - long press enters MIDI update mode
int MIDI_Mode = 1;         // control of LED rows 7 & 8: 1 = Euclidian Mode, 2 = MIDI Mode 3 = Save/Recall Mode

// MIDI Variables
boolean runningStatus = 0;   // use running status after the first midi note to speed tansmission
int activeMIDI = 0;          // which channel is being updated zero referenced (0-3)
int chknob;                  // encoder 1 sets midi channel
int vlknob;                  // encoder 2 sets note velocity (x4)
int ntknob;                  // encoder 3 midi notes (x4)



// load saved midi settings

unsigned int MIDI_sets[4][2] = {
  {EEPROM.read(17), EEPROM.read(21)},
  {EEPROM.read(18), EEPROM.read(22)},
  {EEPROM.read(19), EEPROM.read(23)},
  {EEPROM.read(20), EEPROM.read(24)}
};
int MIDI_chan = EEPROM.read(13);   // MIDI channel (1-16)



//--------------NEW Preset variables------------------------------
int Preset_number = 0;    // preset to read or write
int presetStart = 50;     // starting memory location for preset storage
int Preset_offset;        // starting offset for  a presets parameters in EEPROM
int Previous_Mode = 1;    // mode to return to after write or read
int Write_delay = 3000;   // wait for long press to write or read
int psknob;               // preset variable for encoder read
int Preset_size = 25;     // how many bytes is a preset


/*
  The ADC now runs free-run reading only the Encoder switches
  Clock IN and Reset IN are now digital inputs
*/
int aval; // value from auto cycle ADC interupt routine

int btknob; // LED brightness encoder read
int setBrightness; //LED matrix brightness

boolean gateSwitch = 0; // Are outputs triggers or gates

//-----------------------------------------------------------------


void setup() {

  // setup ADC and Timer 2
  setupADC();

  /*
    Iterrupt sevice routine on CLOCK IN
    Allows Gate Mode to reset during LED Matrix update routine
    LED update is slow - approx 8ms
  */

  PCICR |= 0b00000010;    // turn on port c
  PCMSK1 |= 0b00000001;   // turn on pin PC0/ AO/ Clock in


  /*
    The MAX72XX is in power-saving mode on startup,
     we have to do a wakeup call
  */

  //delay(1000);
  lc.shutdown(0, true);
  /* Set the brightness to a medium values */
  lc.setIntensity(0, brightness);
  /* and clear the display */
  lc.clearDisplay(0);

  setBrightness = EEPROM.read(25); // read saved brightness and set
  lc.setIntensity(0, setBrightness);

  //wakeanim();

  //
  //	IF THE NANO FAILS TO BOOT (DUE TO CORRUPT EEPROM), COMMENT OUT FOLLOWING IF AND END BRACKET
  //	TO FORCE NEW EEPROM WRITE.
  //

  if ((EEPROM.read(1) > 16) || (EEPROM.read(2) > 16) || (EEPROM.read(3) > 16) || (EEPROM.read(4) > 16) || (EEPROM.read(5) > 16) || (EEPROM.read(6) > 16)) { // if eprom is blank / corrupted, write some startup amounts
    // write a 0 to all 512 bytes of the EEPROM
    for (int i = 0; i < 1024; i++)
      EEPROM.write(i, 50);

    EEPROM.write(1, 16);
    EEPROM.write(2, 4);
    EEPROM.write(3, 12);
    EEPROM.write(4, 6);
    EEPROM.write(5, 8);
    EEPROM.write(6, 5);
    EEPROM.write(7, 16);
    EEPROM.write(8, 2);
    EEPROM.write(9, 2);
    EEPROM.write(10, 16);
    EEPROM.write(11, 2);
    EEPROM.write(12, 2);
  }

  digitalWrite(enc1a, HIGH);       // turn on pullup resistor
  digitalWrite(enc1b, HIGH);       // turn on pullup resistor
  digitalWrite(enc2a, HIGH);       // turn on pullup resistor
  digitalWrite(enc2b, HIGH);       // turn on pullup resistor
  digitalWrite(enc3a, HIGH);       // turn on pullup resistor
  digitalWrite(enc3b, HIGH);       // turn on pullup resistor

  if (debug == 2) {
    Serial.begin(9600);
  }

  for (a = 16; a < 20; a++) {
    pinMode(a, OUTPUT);
  }


  //  initialise beat holders

  for (int a = 0; a < channels; a++) {
    beat_holder[a] = euclid(channelbeats[a][0], channelbeats[a][1], channelbeats[a][3]);
  }


  // new pin assignments for digital INs and PWM
  pinMode(A0, INPUT);  // CLOCK IN
  pinMode(A1, INPUT);  // RESET IN
  pinMode(12, INPUT);  // Gate switch
  pinMode(13, OUTPUT);  // unused
  pinMode(11, OUTPUT); // Density PWM OUT

  digitalWrite(13, 0);
  Sync();

  //start MIDI
  Serial.begin(31250); // setup serial for MIDI on TX PIN
  //Serial.begin(9600);  // print debug


  // DEBUG EEPROM Write DUMP EEPROM CONTENT TO SERIAL PRINT
  /*
    int dbgloop = 0;

    Serial.print("EEPROM Dump");
    Serial.println();
    for (dbgloop = 1; dbgloop < 25; dbgloop++)
    {
      Serial.print(dbgloop);
      Serial.print(" - ");
      Serial.print(EEPROM.read(dbgloop));
      Serial.println();
    }
    Serial.print("END Dump");
    Serial.println();
  */

}




// --------------------------------------------------------MAIN LOOP--------------------------------------

void loop()
{

  /*
    What's in the loop:
    Update time variable
    Check to see if it is time go go to sleep
    Changes routine - update beat_holder when channelbeats changes - triggered by changes == true
    Trigger routines - on trigget update displays and pulse
    Read encoders
    Read switches
  */

  time = millis();

  // COPY OVER N, K & ) VARIABLES FOR EASE OF CODE READING
  nn = channelbeats[active_channel][0];
  kk = channelbeats[active_channel][1];
  oo = channelbeats[active_channel][3];

  // Self clocking on startup
  if (debug > 0 && time - last_sync > 250)
  {
    Sync();
  }

  // SLEEP ROUTINE
  if (sleep == false && time - last_sync > 1000000)
  {
    sleepanim();
    lc.shutdown(0, true);
    sleep = true;
  }


  // UPDATE BEAT HOLDER WHEN KNOBS ARE MOVED

  if (changes > 0) {
    beat_holder[active_channel] = euclid(nn, kk, oo);
    lc.setRow(0, active_channel * 2 + 1, 0);//clear active row
    lc.setRow(0, active_channel * 2, 0);//clear line above active row

    if (changes == 1) {  // 1 = K changes - display beats in the active channel
      for (a = 0; a < 8; a++) {
        if (bitRead(beat_holder[active_channel], nn - 1 - a) == 1 && a < nn) {
          lc.setLed(0, active_channel * 2, 7 - a, true);
        }
        if (bitRead(beat_holder[active_channel], nn - 1 - a - 8) == 1 && a + 8 < nn) {
          lc.setLed(0, active_channel * 2 + 1, 7 - a, true);
        }
      }
    }

    if (changes == 2) { // 2 = N changes, display total length of beat
      for (a = 0; a < 8; a++) {
        if (a < nn) {
          lc.setLed(0, active_channel * 2, 7 - a, true);
        }
        if (a + 8 < nn) {
          lc.setLed(0, active_channel * 2 + 1, 7 - a, true);
        }
      }
    }

    if (changes == 3) {  // 3 = Offset changes - display beats in the active channel
      for (a = 0; a < 8; a++) {
        if (bitRead(beat_holder[active_channel], nn - 1 - a) == 1 && a < nn) {
          lc.setLed(0, active_channel * 2, 7 - a, true);
        }
        if (bitRead(beat_holder[active_channel], nn - 1 - a - 8) == 1 && a + 8 < nn) {
          lc.setLed(0, active_channel * 2 + 1, 7 - a, true);
        }
      }
    }

    changes = 0;
    last_changed = time;
  }

  // Launch ADC to read encoder switches in background
  ADCSRA |= B01000000;


  // read Pulse/Gate switch
  // when low pulses, when high outputs stay high untill CLOCK in goes low
  gateSwitch = PINB & 16;


  // CLOCK IN
  newpulse = PINC & 1; // Direct port read of DIG IN C0

  if (newpulse > oldpulse)
  {
    debug = 0;  // turn off internal clock if external clock received
    Sync();     // advance sequencer
  }
  
/*
  if (newpulse < oldpulse && gateSwitch != 0)     //Clock signal goes low
  {
    PORTC = PORTC & B11000011;  //Turn off Pulses in GATE OUT mode
    pulses_active = false;
  }
*/

  oldpulse = newpulse;



  /*  READ KNOBS and Switches
    Switch command directs program depending on encoder mode
    Mode it racked by the MIDI_Mode variable

    What to read knobs and switches for,
    MIDI_Mode
    1 = Euclidian Sequencer
    2 = MIDI setup
    3 = Store/Recall settings
    4 = Briteness adjust
  */

  // read knobs
  switch (MIDI_Mode) {
    case 1:
      E_read_knobs();
      break;
    case 2:
      M_read_knobs();
      break;
    case 3:
      P_read_knobs();
      break;
    case 4:
      B_read_knobs();
      break;
  }

  //  Select which Switch pressed

  // ADC is in freerun mode and ACD result is in ADCL/ADCH registers
  // read low first
  channel_switch_read = ADCL | (ADCH << 8);

  if (channel_switch_read < 100) {
    channel_switch = 3;					//	Nothing Pressed. (3)
  };
  if (channel_switch_read > 100 && channel_switch_read < 200) {
    channel_switch = 1;					//	Button 2, Density (1)
  };
  if (channel_switch_read > 200 && channel_switch_read < 400) {
    channel_switch = 0;					//	Button 1, Length Pressed. (0) SELECT ACTIVE MIDI note (Euclidian channel)
    M_advance_MIDI();   // advance sequencer channel // ********************MOVE TO M_read_switches*****************************
  };
  if (channel_switch_read > 400) {
    channel_switch = 2;					//	Button 3, Offset Pressed. (2)
  };

  if (sleep == true) {
    if (channel_switch != 3) // wake up display if you push a button
    {
      if (sleep == true)// wake up routine & animation
      {
        lc.shutdown(0, false);
        sleep = false;
        last_sync = time;
        //      wakeanim();
      }
    }
  }

  // direct to mode routine - Euclidian, MIDI, Recall/Store
  switch (MIDI_Mode) {
    case 1:
      E_read_switches();    // switches set active channel
      break;
    case 2:
      M_read_switches();    // switches set which channel MIDI settings are updated
      break;
    case 3:
      P_read_switches();    // switches select store/recall preset and read or write
      break;
    case 4:
      //unused              // no switch routing in brightness mode
      break;
  }



  // ENABLE RESET BUTTON
  reset_button = PINC & 2; // direct digital read of port C pin 1

  if (reset_button != 0 && channelbeats[0][2] > 0) {
    for (a = 0; a < channels; a++) {
      channelbeats[a][2] = 0;
    }
  }



  // TURN OFF ANY LIGHTS THAT ARE ON
  if (time - last_sync > pulse_length && lights_active == true) {
    for (a = 0; a < channels; a++) {
      if (M_displayDelay == 1) {   // don't update if rows are in use by MIDI or Preset
        lc.setLed(0, 7, 5 - (a * 2), false);
        lc.setLed(0, 7, 4, false); // spare pin flash
        lc.setRow(0, 7, 0);
        lights_active = false;
      }
    }
  }

  // FINISH ANY PULSES THAT ARE ACTIVE - PULSES LAST 1/4 AS LONG AS LIGHTS
  if (time - last_sync > (pulse_length / 4) && pulses_active == true) {
    if (gateSwitch == 0)
    {
      PORTC = PORTC & B11000011;  //Turn off Pulses
    }
    for (a = 0; a < channels; a++) {
      playMIDIOff(); // note offs
      runningStatus = 1;
    }
    a = 3;
    playMIDIOff(); // note off for offbeat
    pulses_active = false;
  }
  runningStatus = 0;
}


// -----------END MAIN---------------------------------



// Euclid calculation function

unsigned int euclid(int n, int k, int o) { // inputs: n=total, k=beats, o = offset
  int pauses = n - k;
  int pulses = k;
  int offset = o;
  int steps = n;
  int per_pulse = pauses / k;
  int remainder = pauses % pulses;
  unsigned int workbeat[n];
  unsigned int outbeat;
  uint16_t outbeat2;
  unsigned int working;
  int workbeat_count = n;
  int a;
  int b;
  int trim_count;

  for (a = 0; a < n; a++) { // Populate workbeat with unsorted pulses and pauses
    if (a < pulses) {
      workbeat[a] = 1;
    }
    else {
      workbeat[a] = 0;
    }
  }

  if (per_pulse > 0 && remainder < 2) { // Handle easy cases where there is no or only one remainer
    for (a = 0; a < pulses; a++) {
      for (b = workbeat_count - 1; b > workbeat_count - per_pulse - 1; b--) {
        workbeat[a] = ConcatBin(workbeat[a], workbeat[b]);
      }
      workbeat_count = workbeat_count - per_pulse;
    }

    outbeat = 0; // Concatenate workbeat into outbeat - according to workbeat_count
    for (a = 0; a < workbeat_count; a++) {
      outbeat = ConcatBin(outbeat, workbeat[a]);
    }

    if (offset > 0) {
      outbeat2 = rightRotate(offset, outbeat, steps); // Add offset to the step pattern
    }
    else {
      outbeat2 = outbeat;
    }

    return outbeat2;
  }

  else {
    if (pulses == 0) {
      pulses = 1;  //	Prevent crashes when k=0 and n goes from 0 to 1
    }
    int groupa = pulses;
    int groupb = pauses;
    int iteration = 0;
    if (groupb <= 1) {
    }

    while (groupb > 1) { //main recursive loop
      /*Serial.print("Pulses: ");		//	Help debug crash
        Serial.println(pulses);
        Serial.print("Pauses: ");
        Serial.println(pauses);
        Serial.print("Iterations: ");
        Serial.println(iteration);*/
      if (groupa > groupb) { // more Group A than Group B
        int a_remainder = groupa - groupb; // what will be left of groupa once groupB is interleaved
        trim_count = 0;
        for (a = 0; a < groupa - a_remainder; a++) { //count through the matching sets of A, ignoring remaindered
          workbeat[a] = ConcatBin(workbeat[a], workbeat[workbeat_count - 1 - a]);
          trim_count++;
        }
        workbeat_count = workbeat_count - trim_count;

        groupa = groupb;
        groupb = a_remainder;
      }

      else if (groupb > groupa) { // More Group B than Group A
        int b_remainder = groupb - groupa; // what will be left of group once group A is interleaved
        trim_count = 0;
        for (a = workbeat_count - 1; a >= groupa + b_remainder; a--) { //count from right back through the Bs
          workbeat[workbeat_count - a - 1] = ConcatBin(workbeat[workbeat_count - a - 1], workbeat[a]);

          trim_count++;
        }
        workbeat_count = workbeat_count - trim_count;
        groupb = b_remainder;
      }

      else if (groupa == groupb) { // groupa = groupb
        trim_count = 0;
        for (a = 0; a < groupa; a++) {
          workbeat[a] = ConcatBin(workbeat[a], workbeat[workbeat_count - 1 - a]);
          trim_count++;
        }
        workbeat_count = workbeat_count - trim_count;
        groupb = 0;
      }

      else {
        //Serial.println("ERROR");
      }
      iteration++;
    }

    outbeat = 0; // Concatenate workbeat into outbeat - according to workbeat_count
    for (a = 0; a < workbeat_count; a++) {
      outbeat = ConcatBin(outbeat, workbeat[a]);
    }

    if (offset > 0) {
      outbeat2 = rightRotate(offset, outbeat, steps); // Add offset to the step pattern
    }
    else {
      outbeat2 = outbeat;
    }

    return outbeat2;
  }
}

//------------------end euclidian math-------------------------


/*Function to right rotate n by d bits*/

uint16_t rightRotate(int shift, uint16_t value, uint8_t pattern_length) {
  uint16_t mask = ((1 << pattern_length) - 1);
  value &= mask;
  return ((value >> shift) | (value << (pattern_length - shift))) & mask;
}

// Function to find the binary length of a number by counting bitwise
int findlength(unsigned int bnry) {
  boolean lengthfound = false;
  int length = 1; // no number can have a length of zero - single 0 has a length of one, but no 1s for the sytem to count
  for (int q = 32; q >= 0; q--) {
    int r = bitRead(bnry, q);
    if (r == 1 && lengthfound == false) {
      length = q + 1;
      lengthfound = true;
    }
  }
  return length;
}

// Function to concatenate two binary numbers bitwise
unsigned int ConcatBin(unsigned int bina, unsigned int binb) {
  int binb_len = findlength(binb);
  unsigned int sum = (bina << binb_len);
  sum = sum | binb;
  return sum;
}


//------------------------------------------------------------------------------------


// routine triggered by each beat
void Sync()
{
  if (sleep == true)// wake up routine & animation
  {
    lc.shutdown(0, false);
    sleep = false;
    wakeanim();
  }


  // Cycle through channels

  /*
    Do the outputs first to speed response time
    the slower LCD commands are done later

    New routine runs through and marks which gates should go high, then outputs them as a group
    then outputs Offbeat gate
    then outputs MIDI
  */

  int TEMP1 = channelbeats[0][2]; //store location to reset loop
  int TEMP2 = channelbeats[1][2];
  int TEMP3 = channelbeats[2][2];
  byte gateOut = 0;   // byte to write to port C - Pulses on pins A2-A5
  // byte offBeatOut = 0;// byte to write to port C

  // DO OUTPUTS
  for (a = 0; a < channels; a++)
  {
    read_head = channelbeats[a][0] - channelbeats[a][2] - 1;

    // turn on pulses on channels where a beat is present
    if (bitRead(beat_holder[a], read_head) == 1)
    {
      bitSet(gateOut, a + 2); // Mark which channels are high to output as a set
    }
    // set OFFBEAT pulse
    if (bitRead(beat_holder[a], read_head) == 0 && a == 0) { // only relates to first channel
      bitSet(gateOut, 5);
    }
    pulses_active = true;
    lights_active = true;

    // move counter to next position, ready for next pulse
    channelbeats[a][2]++;
    if (channelbeats[a][2] >= channelbeats[a][0]) {
      channelbeats[a][2] = 0;
    }
  }

  /*
     PORT C
     7 ANALOG ENCODER SWITCH IN
     6 UNUSED
     5 OFFBEAT OUT
     4 CH 3 OUT
     3 CH 2 OUT
     2 CH 1 OUT
     1 RESET IN
     0 CLOCK IN
  */

  PORTC = PORTC | gateOut; // Write to port - Set Gates HIGH

  // Density output


  // Set number for DENSITY OUT
  byte dense = gateOut;
  bitClear (dense, 5);           // clear offbeat bit from count
  dense = dense >> 2;          // shuffle to lowest 3 bits
  byte DensityOut = dense * 32;  // scale to 0-255
  //analogWrite(11, DensityOut);
  OCR2A = DensityOut;

  // restore beginning position and run through MIDI outs
  channelbeats[0][2] = TEMP1;
  channelbeats[1][2] = TEMP2;
  channelbeats[2][2] = TEMP3;

  // Send MIDI note_ons

  runningStatus = 0; // first note sent has status bit
  for (a = 0; a < channels; a++) {

    if (bitRead(gateOut, (a + 2)) == 1)
    {
      playMIDINote();    // play a note
      runningStatus = 1; // midi note has been played, don't need to send ch # any more
    }
    // move counter to next position, ready for next pulse
    channelbeats[a][2]++;
    if (channelbeats[a][2] >= channelbeats[a][0]) {
      channelbeats[a][2] = 0;
    }
  }

  if (bitRead(gateOut, 5) == 1) // offBeat for CH 1
  {
    a = 3;
    playMIDINote();
  }
  runningStatus = 0;

  // DO DISPLAY

  // digitalWrite(13, 1); // *************************************

  // restore beginning position
  channelbeats[0][2] = TEMP1;
  channelbeats[1][2] = TEMP2;
  channelbeats[2][2] = TEMP3;

  // clear bottom row
  //lc.setRow(0,7,0);
  if (M_displayDelay == 1) {       // don't do it if the row is being used by MIDI
    //   lc.setLed(0, 7, 7, true);      // CL light on every beat
    if (masterclock % 2 == 0) { // tick bottom left corner on and off with clock
      lc.setLed(0, 7, 7, true);
    }
    else {
      lc.setLed(0, 7, 7, false);
    }
  }

  for (a = 0; a < channels; a++) {
    read_head = channelbeats[a][0] - channelbeats[a][2] - 1;
    if (a != active_channel || time - last_changed > display_update)     // don't clear or draw cursor if channel is being changed
    {
      lc.setRow(0, a * 2, 0);//clear line above active row

      if (channelbeats[a][2] < 8) {
        for (int c = 0; c < 8; c++) {
          if (bitRead(beat_holder[a], channelbeats[a][0] - 1 - c) == 1 && c < channelbeats[a][0]) {
            lc.setLed(0, a * 2, 7 - c, true);
          }
        }
      }
      else {
        for (int c = 8; c < 16; c++) {
          if (bitRead(beat_holder[a], channelbeats[a][0] - 1 - c) == 1 && c < channelbeats[a][0]) {
            lc.setLed(0, a * 2, 15 - c, true);
          }
        }
      }

      lc.setRow(0, a * 2 + 1, 0);//clear active row
      // draw cursor
      if (channelbeats[a][2] < 8) {
        lc.setLed(0, a * 2 + 1, 7 - channelbeats[a][2], true); // write cursor less than 8
      }
      else if (channelbeats[a][2] >= 8 && channelbeats[a][2] < 16) {
        lc.setLed(0, a * 2 + 1, 15 - channelbeats[a][2], true); // write cursor more than 8
      }
    }

    // turn on LED on channels where a beat is present
    if (bitRead(beat_holder[a], read_head) == 1) {

      //lc.setLed(0, 7, 5 - (b * 2), true); // bottom row flash
      if (M_displayDelay == 1) {
        if (a == 0) {
          lc.setLed(0, 7, 5, true);
        }
        if (a == 1) {
          lc.setLed(0, 7, 2, true);
        }
        if (a == 2) {
          lc.setLed(0, 7, 0, true);
        }
        pulses_active = true;
        lights_active = true;

      }
    }
    // LED for spare output for the first channel
    if (bitRead(beat_holder[a], read_head) == 0 && a == 0) { // only relates to first channel

      if (M_displayDelay == 1) {
        lc.setLed(0, 7, 4, true); // bottom row flash
      }
      pulses_active = true;
      lights_active = true;
    }

    // move counter to next position, ready for next pulse
    channelbeats[a][2]++;
    if (channelbeats[a][2] >= channelbeats[a][0]) {
      channelbeats[a][2] = 0;
    }
  }

  //  digitalWrite(13, 0); // *****************************************

  masterclock++;
  if (masterclock >= 16) {
    masterclock = 0;
  }

  looptracker++;
  //   length = ((time - last_sync) / 5);
  last_sync = time;
}

//----------------------

/*ENCODER READ ROUTINES

  3 functions to read each encoder
  returns +1, 0 or -1 dependent on direction
  Contains no internal debounce, so calls should be delayed
  K=SDensity
  L=Length
  O=Offset
*/

int EncodeReadK() {

  int result = 0;
  if (EncK.read() == 0) {
    EncK.write(0);
    result = 0;
  }
  else if (EncK.read() < 0) {
    result = -1;
    EncK.write(0);
  }
  else if (EncK.read() > 0) {
    result = 1;
    EncK.write(0);
  }
  return result;
}


int EncodeReadN() {
  int result = 0;
  if (EncN.read() == 0) {
    EncN.write(0);
    result = 0;
  }
  else if (EncN.read() < 0) {
    result = -1;
    EncN.write(0);
  }
  else if (EncN.read() > 0) {
    result = 1;
    EncN.write(0);
  }
  return result;
}


int EncodeReadO() {
  int result = 0;
  if (EncO.read() == 0) {
    EncO.write(0);
    result = 0;
  }
  else if (EncO.read() < 0) {
    result = -1;
    EncO.write(0);
  }
  else if (EncO.read() > 0) {
    result = 1;
    EncO.write(0);
  }
  return result;
}


// Matrix LED wake-up and sleep animation

void wakeanim() {
  for (a = 4; a > 0; a--) {
    lc.setRow(0, a, 255);
    lc.setRow(0, 8 - a, 255);
    delay(100);
    lc.setRow(0, a, 0);
    lc.setRow(0, 8 - a, 0);
  }
}
void sleepanim() {
  for (a = 0; a < 4; a++) {
    lc.setRow(0, a, 255);
    lc.setRow(0, 8 - a, 255);
    delay(200);
    lc.setRow(0, a, 0);
    lc.setRow(0, 8 - a, 0);
  }
}


//MIDI out routines

// routines check if <MIDI_chan> = 0 which is OFF and returns without doing anything
// if MIDI is turned off

// MIDI note_on
void playMIDINote()
{
  if (MIDI_chan != 0)
  {
    if (runningStatus == 0)
    {
      byte noteOnStatus = 0x90 + (MIDI_chan - 1);
      Serial.write(noteOnStatus); // note_on and MIDI channel
    }
    Serial.write(MIDI_sets[a][1]);// note number
    Serial.write(MIDI_sets[a][0]);// velocity
  }
}

//MIDI note_off
void playMIDIOff()
{
  if (MIDI_chan != 0)
  {
    if (runningStatus == 0)
    {
      byte noteOffStatus = 0x80 + (MIDI_chan - 1);
      Serial.write(noteOffStatus); // note_off and MIDI channel
    }
    Serial.write(MIDI_sets[a][1]); // note number
    Serial.write(0);               // velocity
  }
}

//----------------------------

//GATE RESET ROUTINE
ISR(PCINT1_vect)
{
  if (gateSwitch != 0)     //Clock signal goes low
  {
    PORTC = PORTC & B11000011;  //Turn off Pulses in GATE OUT mode
  }
}


// ------END---------
